/* Copyright (C) Øyvind Kolås, warning this code originates in glade, could
 * with benefit be rewritten
 */

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>

#include <gdk/gdkkeysyms.h>
#include <gtk/gtk.h>

#include "color_correct.h"

#include <libgimp/gimp.h>
#include <libgimp/gimpui.h>
#include "ui.h"
#include "data.h"
#include "colorconv.h"


static void
on_prev_image_toggled (GtkToggleButton *togglebutton,
                       gpointer         user_data);

static void
on_vectorscope_toggled (GtkToggleButton *togglebutton,
                        gpointer         user_data);

static void
on_lockgray_toggled (GtkToggleButton *togglebutton,
                     gpointer         user_data);

static gboolean
on_black_drawingarea_expose_event (GtkWidget      *widget,
                                   GdkEventExpose *event,
                                   gpointer        user_data);

static gboolean
on_black_drawingarea_configure_event (GtkWidget         *widget,
                                      GdkEventConfigure *event,
                                      gpointer           user_data);

static gboolean
on_black_drawingarea_button_press_event (GtkWidget      *widget,
                                         GdkEventButton *event,
                                         gpointer        user_data);

static gboolean
on_black_drawingarea_motion_notify_event (GtkWidget      *widget,
                                          GdkEventMotion *event,
                                          gpointer        user_data);

static gboolean
on_gray_drawingarea_expose_event (GtkWidget      *widget,
                                  GdkEventExpose *event,
                                  gpointer        user_data);

static gboolean
on_gray_drawingarea_configure_event (GtkWidget         *widget,
                                     GdkEventConfigure *event,
                                     gpointer           user_data);

static gboolean
on_gray_drawingarea_motion_notify_event (GtkWidget      *widget,
                                         GdkEventMotion *event,
                                         gpointer user_data);

static gboolean
on_gray_drawingarea_button_press_event (GtkWidget      *widget,
                                        GdkEventButton *event,
                                        gpointer        user_data);

static gboolean
on_white_drawingarea_button_press_event (GtkWidget      *widget,
                                         GdkEventButton *event,
                                         gpointer        user_data);

static gboolean
on_white_drawingarea_configure_event (GtkWidget         *widget,
                                      GdkEventConfigure *event,
                                      gpointer           user_data);

static gboolean
on_white_drawingarea_expose_event (GtkWidget      *widget,
                                   GdkEventExpose *event,
                                   gpointer        user_data);

static gboolean
on_white_drawingarea_motion_notify_event (GtkWidget      *widget,
                                          GdkEventMotion *event,
                                          gpointer        user_data);

static gboolean
on_vectorscope_drawingarea_expose_event (GtkWidget      *widget,
                                         GdkEventExpose *event,
                                         gpointer        user_data);

static gboolean
on_vectorscope_drawingarea_configure_event (GtkWidget         *widget,
                                            GdkEventConfigure *event,
                                            gpointer           user_data);


GtkWidget *vectorscope_drawingarea;
GtkWidget *saturation_spin;
GtkWidget *lockgray;
GtkWidget *prev_image;
GtkWidget *black_drawingarea;
GtkWidget *bu;
GtkWidget *bv;
GtkWidget *gray_drawingarea;
GtkWidget *gu;
GtkWidget *gv;
GtkWidget *white_drawingarea;
GtkWidget *wu;
GtkWidget *wv;

static GtkWidget *vectorscope_toggle;

GtkWidget *colorbal_ui (void)
{
  GtkTooltips *tooltips;

  GtkWidget *colorbal_vbox;
  GtkWidget *colors_tophbox;
  GtkWidget *vectorscope_frame;
  GtkWidget *vbox15;
  GtkWidget *saturation_hbox;
  GtkWidget *saturation_label;
  GtkWidget *saturation_scale;
  GtkObject *saturation_adj;
  GtkWidget *points_hbox;
  GtkWidget *black_frame;
  GtkWidget *black_vbox;
  GtkWidget *black_values;
  GtkObject *bu_adj;
  GtkObject *bv_adj;
  GtkWidget *gray_frame;
  GtkWidget *gray_vbox;
  GtkWidget *gray_values;
  GtkObject *gu_adj;
  GtkObject *gv_adj;

  GtkWidget *white_frame;
  GtkWidget *white_vbox;
  GtkWidget *white_values;
  GtkObject *wv_adj;
  GtkObject *wu_adj;

  tooltips = gtk_tooltips_new ();

  colorbal_vbox = gtk_vbox_new (FALSE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (colorbal_vbox), 2);

  colors_tophbox = gtk_hbox_new (FALSE, 0);
  gtk_box_pack_start (GTK_BOX (colorbal_vbox), colors_tophbox, TRUE, TRUE,
            0);

  vectorscope_frame = gtk_frame_new (_("vectorscope"));

  gtk_box_pack_start (GTK_BOX (colors_tophbox), vectorscope_frame, FALSE,
            FALSE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (vectorscope_frame), 2);

  vectorscope_drawingarea = gtk_drawing_area_new ();

  gtk_container_add (GTK_CONTAINER (vectorscope_frame),
             vectorscope_drawingarea);
  gtk_widget_set_size_request (vectorscope_drawingarea, 128, 128);
  gtk_tooltips_set_tip (tooltips, vectorscope_drawingarea,
              _
              ("this scope shows the color distribution in the image"),
              NULL);
  gtk_widget_set_events (vectorscope_drawingarea, GDK_EXPOSURE_MASK);

  vbox15 = gtk_vbox_new (FALSE, 0);
  gtk_box_pack_start (GTK_BOX (colors_tophbox), vbox15, TRUE, TRUE, 0);

  prev_image = gtk_check_button_new_with_label (_("Preview in image"));
  gtk_box_pack_start (GTK_BOX (vbox15), prev_image, FALSE, FALSE, 0);
  gtk_tooltips_set_tip (tooltips, prev_image,
              _
              ("use the image's own drawing area for a preview as well"),
              NULL);

  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (prev_image),
                  pp.prev_in_image);

  g_signal_connect (GTK_OBJECT (prev_image), "toggled",
            GTK_SIGNAL_FUNC (on_prev_image_toggled), NULL);


  vectorscope_toggle =
    gtk_check_button_new_with_label (_("Enable vectorscope"));
  gtk_box_pack_start (GTK_BOX (vbox15), vectorscope_toggle, FALSE, FALSE,
            0);
  gtk_tooltips_set_tip (tooltips, vectorscope_toggle,
              _("update vectorscope together with preview"),
              NULL);

  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (vectorscope_toggle),
                  pp.vectorscope_enable);
  gtk_widget_set_sensitive (vectorscope_toggle, pp.prev_in_image);

  g_signal_connect (GTK_OBJECT (vectorscope_toggle), "toggled",
            GTK_SIGNAL_FUNC (on_vectorscope_toggled), NULL);


  lockgray = gtk_check_button_new_with_label (_("lock gray point"));
  gtk_box_pack_end (GTK_BOX (vbox15), lockgray, FALSE, FALSE, 0);
  gtk_tooltips_set_tip (tooltips, lockgray,
              _
              ("force the graypoint to be in the middle of black and white point"),
              NULL);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (lockgray), pv.lock_gray);

  saturation_hbox = gtk_hbox_new (FALSE, 0);
  gtk_box_pack_end (GTK_BOX (vbox15), saturation_hbox, TRUE, TRUE, 0);

  saturation_label = gtk_label_new (_("Saturation: "));
  gtk_box_pack_start (GTK_BOX (saturation_hbox), saturation_label, FALSE,
            FALSE, 0);

  saturation_adj = gtk_adjustment_new (1, 0, 4, 0.02, 0.05, 0.05);
  saturation_scale = gtk_hscale_new (GTK_ADJUSTMENT (saturation_adj));

  saturation_spin =
    gtk_spin_button_new (GTK_ADJUSTMENT (saturation_adj), 1, 2);
  gtk_box_pack_start (GTK_BOX (saturation_hbox), saturation_scale, TRUE,
            TRUE, 0);
  gtk_box_pack_start (GTK_BOX (saturation_hbox), saturation_spin, FALSE,
            FALSE, 0);

  gtk_scale_set_digits (GTK_SCALE (saturation_scale), 2);


  points_hbox = gtk_hbox_new (TRUE, 0);
  gtk_box_pack_start (GTK_BOX (colorbal_vbox), points_hbox, FALSE, FALSE,
            0);

  black_frame = gtk_frame_new (_("blackpoint"));
  gtk_box_pack_start (GTK_BOX (points_hbox), black_frame, FALSE, TRUE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (black_frame), 2);

  black_vbox = gtk_vbox_new (FALSE, 0);
  gtk_container_add (GTK_CONTAINER (black_frame), black_vbox);

  black_drawingarea = gtk_drawing_area_new ();
  gtk_box_pack_start (GTK_BOX (black_vbox), black_drawingarea, FALSE, FALSE,
            0);
  gtk_widget_set_size_request (black_drawingarea, 128, 128);
  gtk_widget_set_events (black_drawingarea,
               GDK_EXPOSURE_MASK | GDK_BUTTON1_MOTION_MASK |
               GDK_BUTTON_PRESS_MASK);

  black_values = gtk_hbox_new (FALSE, 0);
  gtk_box_pack_start (GTK_BOX (black_vbox), black_values, FALSE, TRUE, 0);

  bu_adj = gtk_adjustment_new (0, -128, 128, 1, 2, 2);
  bu = gtk_spin_button_new (GTK_ADJUSTMENT (bu_adj), 1, 0);
  gtk_box_pack_start (GTK_BOX (black_values), bu, TRUE, TRUE, 0);
  gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (bu), TRUE);

  bv_adj = gtk_adjustment_new (0, -128, 128, 1, 10, 10);
  bv = gtk_spin_button_new (GTK_ADJUSTMENT (bv_adj), 1, 0);
  gtk_box_pack_start (GTK_BOX (black_values), bv, TRUE, TRUE, 0);
  gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (bv), TRUE);

  gray_frame = gtk_frame_new (_("graypoint"));
  gtk_box_pack_start (GTK_BOX (points_hbox), gray_frame, FALSE, TRUE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (gray_frame), 2);

  gray_vbox = gtk_vbox_new (FALSE, 0);
  gtk_container_add (GTK_CONTAINER (gray_frame), gray_vbox);

  gray_drawingarea = gtk_drawing_area_new ();
  gtk_box_pack_start (GTK_BOX (gray_vbox), gray_drawingarea, FALSE, FALSE,
            0);
  gtk_widget_set_size_request (gray_drawingarea, 128, 128);
  gtk_widget_set_events (gray_drawingarea,
               GDK_EXPOSURE_MASK | GDK_BUTTON1_MOTION_MASK |
               GDK_BUTTON_PRESS_MASK);

  gray_values = gtk_hbox_new (FALSE, 0);
  gtk_box_pack_start (GTK_BOX (gray_vbox), gray_values, FALSE, TRUE, 0);

  gu_adj = gtk_adjustment_new (0, -128, 128, 1, 2, 2);
  gu = gtk_spin_button_new (GTK_ADJUSTMENT (gu_adj), 1, 0);
  gtk_box_pack_start (GTK_BOX (gray_values), gu, TRUE, TRUE, 0);
  gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (gu), TRUE);

  gv_adj = gtk_adjustment_new (0, -128, 128, 1, 10, 10);
  gv = gtk_spin_button_new (GTK_ADJUSTMENT (gv_adj), 1, 0);
  gtk_box_pack_start (GTK_BOX (gray_values), gv, TRUE, TRUE, 0);
  gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (gv), TRUE);

  white_frame = gtk_frame_new (_("whitepoint"));
  gtk_box_pack_start (GTK_BOX (points_hbox), white_frame, FALSE, TRUE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (white_frame), 2);

  white_vbox = gtk_vbox_new (FALSE, 0);
  gtk_container_add (GTK_CONTAINER (white_frame), white_vbox);

  white_drawingarea = gtk_drawing_area_new ();
  gtk_box_pack_start (GTK_BOX (white_vbox), white_drawingarea, FALSE, FALSE,
            0);
  gtk_widget_set_size_request (white_drawingarea, 128, 128);
  gtk_widget_set_events (white_drawingarea,
               GDK_EXPOSURE_MASK | GDK_BUTTON1_MOTION_MASK |
               GDK_BUTTON_PRESS_MASK);

  white_values = gtk_hbox_new (FALSE, 0);
  gtk_box_pack_start (GTK_BOX (white_vbox), white_values, FALSE, TRUE, 0);

  wu_adj = gtk_adjustment_new (0, -128, 128, 1, 2, 2);
  wu = gtk_spin_button_new (GTK_ADJUSTMENT (wu_adj), 1, 0);
  gtk_box_pack_start (GTK_BOX (white_values), wu, TRUE, TRUE, 0);
  gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (wu), TRUE);

  wv_adj = gtk_adjustment_new (0, -128, 128, 1, 10, 10);
  wv = gtk_spin_button_new (GTK_ADJUSTMENT (wv_adj), 1, 0);
  gtk_box_pack_start (GTK_BOX (white_values), wv, TRUE, TRUE, 0);
  gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (wv), TRUE);


  g_signal_connect (GTK_OBJECT (vectorscope_drawingarea), "expose_event",
            GTK_SIGNAL_FUNC
            (on_vectorscope_drawingarea_expose_event), NULL);
  g_signal_connect (GTK_OBJECT (vectorscope_drawingarea), "configure_event",
            GTK_SIGNAL_FUNC
            (on_vectorscope_drawingarea_configure_event), NULL);
  g_signal_connect (GTK_OBJECT (saturation_scale), "button_release_event",
            GTK_SIGNAL_FUNC (on_values_changed), NULL);
  g_signal_connect (GTK_OBJECT (lockgray), "toggled",
            GTK_SIGNAL_FUNC (on_lockgray_toggled), NULL);
  g_signal_connect (GTK_OBJECT (black_drawingarea), "expose_event",
            GTK_SIGNAL_FUNC (on_black_drawingarea_expose_event),
            NULL);
  g_signal_connect (GTK_OBJECT (black_drawingarea), "configure_event",
            GTK_SIGNAL_FUNC (on_black_drawingarea_configure_event),
            NULL);
  g_signal_connect (GTK_OBJECT (black_drawingarea), "button_press_event",
            GTK_SIGNAL_FUNC
            (on_black_drawingarea_button_press_event), NULL);
  g_signal_connect (GTK_OBJECT (black_drawingarea), "motion_notify_event",
            GTK_SIGNAL_FUNC
            (on_black_drawingarea_motion_notify_event), NULL);
  g_signal_connect (GTK_OBJECT (bu), "changed",
            GTK_SIGNAL_FUNC (on_values_changed), NULL);
  g_signal_connect (GTK_OBJECT (bv), "changed",
            GTK_SIGNAL_FUNC (on_values_changed), NULL);
  g_signal_connect (GTK_OBJECT (gray_drawingarea), "expose_event",
            GTK_SIGNAL_FUNC (on_gray_drawingarea_expose_event),
            NULL);
  g_signal_connect (GTK_OBJECT (gray_drawingarea), "configure_event",
            GTK_SIGNAL_FUNC (on_gray_drawingarea_configure_event),
            NULL);
  g_signal_connect (GTK_OBJECT (gray_drawingarea), "motion_notify_event",
            GTK_SIGNAL_FUNC
            (on_gray_drawingarea_motion_notify_event), NULL);
  g_signal_connect (GTK_OBJECT (gray_drawingarea), "button_press_event",
            GTK_SIGNAL_FUNC
            (on_gray_drawingarea_button_press_event), NULL);
  g_signal_connect (GTK_OBJECT (gu), "changed",
            GTK_SIGNAL_FUNC (on_values_changed), NULL);
  g_signal_connect (GTK_OBJECT (gv), "changed",
            GTK_SIGNAL_FUNC (on_values_changed), NULL);
  g_signal_connect (GTK_OBJECT (saturation_spin), "changed",
            GTK_SIGNAL_FUNC (on_values_changed), NULL);
  g_signal_connect (GTK_OBJECT (white_drawingarea), "button_press_event",
            GTK_SIGNAL_FUNC
            (on_white_drawingarea_button_press_event), NULL);
  g_signal_connect (GTK_OBJECT (white_drawingarea), "configure_event",
            GTK_SIGNAL_FUNC (on_white_drawingarea_configure_event),
            NULL);
  g_signal_connect (GTK_OBJECT (white_drawingarea), "expose_event",
            GTK_SIGNAL_FUNC (on_white_drawingarea_expose_event),
            NULL);
  g_signal_connect (GTK_OBJECT (white_drawingarea), "motion_notify_event",
            GTK_SIGNAL_FUNC
            (on_white_drawingarea_motion_notify_event), NULL);
  g_signal_connect (GTK_OBJECT (wu), "changed",
            GTK_SIGNAL_FUNC (on_values_changed), NULL);
  g_signal_connect (GTK_OBJECT (wv), "changed",
            GTK_SIGNAL_FUNC (on_values_changed), NULL);

  return colorbal_vbox;
}

static void
on_vectorscope_toggled (GtkToggleButton *togglebutton,
                        gpointer         user_data)
{
  pp.vectorscope_enable = gtk_toggle_button_get_active (togglebutton);
  if (pp.vectorscope_enable) {
    update_image_preview ();
  } else {
    gtk_widget_queue_draw (vectorscope_drawingarea);
  }
}

static void
on_prev_image_toggled (GtkToggleButton *togglebutton,
                       gpointer         user_data)
{
  pp.prev_in_image = gtk_toggle_button_get_active (togglebutton);

  gtk_widget_set_sensitive (vectorscope_toggle, pp.prev_in_image);

  update_image_preview ();
  gtk_widget_queue_draw (vectorscope_drawingarea);
}



static void
on_lockgray_toggled (GtkToggleButton *togglebutton,
                     gpointer         user_data)
{
  pv.lock_gray = gtk_toggle_button_get_active (togglebutton);
}

static gchar *black_bg = NULL;
static gchar *gray_bg = NULL;
static gchar *white_bg = NULL;

static void make_pointbg (gchar *bg,
                          gint   Y)
{
  int x, y, R, G, B, U, V;

  for (x = 0; x < 128; x++)
    for (y = 0; y < 128; y++) {
      U = ((x - 64) * 2) + 128;
      V = ((64 - y) * 2) + 128;
      YUV2RGB (Y, U, V, R, G, B);
      bg[y * (128 * 3) + x * 3] = R;
      bg[y * (128 * 3) + x * 3 + 1] = G;
      bg[y * (128 * 3) + x * 3 + 2] = B;
    }
}

static gboolean
on_black_drawingarea_configure_event (GtkWidget         *widget,
                                      GdkEventConfigure *event,
                                      gpointer           user_data)
{
  if (!black_bg) {
    black_bg = g_malloc (128 * 128 * 3);
    make_pointbg (black_bg, 0);
  }
  return FALSE;
}

static void update_point (GtkWidget *widget,
                          gint      *u, gint    *v,
                          gdouble    x, gdouble  y)
{
  x = (x - 64) * 2;
  y = (64 - y) * 2;

  *u = x;
  *v = y;

  if (u != &pv.gu) {
    if (pv.lock_gray) {
      pv.gu = (pv.bu + pv.wu) / 2;
      pv.gv = (pv.bv + pv.wv) / 2;

      gtk_widget_queue_draw (gray_drawingarea);
    }
  }

}

static gboolean
on_black_drawingarea_expose_event (GtkWidget      *widget,
                                   GdkEventExpose *event,
                                   gpointer        user_data)
{
  gdk_draw_rgb_image (widget->window, widget->style->white_gc,
            0, 0, 128, 128, GDK_RGB_DITHER_NONE, black_bg,
            128 * 3);

  gdk_draw_rectangle (widget->window, widget->style->white_gc,
            TRUE, (pv.bu / 2) + 64, 0, 1, 128);

  gdk_draw_rectangle (widget->window, widget->style->white_gc,
            TRUE, 0, 64 - (pv.bv / 2), 128, 1);

  return FALSE;
}

static gboolean
on_black_drawingarea_button_press_event (GtkWidget      *widget,
                                         GdkEventButton *event,
                                         gpointer        user_data)
{
  update_point (widget, &pv.bu, &pv.bv, event->x, event->y);
  update_values ();

  gtk_widget_queue_draw (widget);

  return FALSE;
}


static gboolean
on_black_drawingarea_motion_notify_event (GtkWidget      *widget,
                                          GdkEventMotion *event,
                                          gpointer        user_data)
{
  return on_black_drawingarea_button_press_event (widget,
                          (GdkEventButton *) event,
                          user_data);
}


static gboolean
on_gray_drawingarea_expose_event (GtkWidget      *widget,
                                  GdkEventExpose *event,
                                  gpointer        user_data)
{
  gdk_draw_rgb_image (widget->window, widget->style->white_gc,
            0, 0, 128, 128, GDK_RGB_DITHER_NONE, gray_bg,
            128 * 3);

  gdk_draw_rectangle (widget->window, widget->style->black_gc,
            TRUE, (pv.gu / 2) + 64, 0, 1, 128);

  gdk_draw_rectangle (widget->window, widget->style->black_gc,
            TRUE, 0, 64 - (pv.gv / 2), 128, 1);
  return FALSE;
}


static gboolean
on_gray_drawingarea_configure_event (GtkWidget         *widget,
                                     GdkEventConfigure *event,
                                     gpointer           user_data)
{
  if (!gray_bg)
    {
      gray_bg = g_malloc (128 * 128 * 3);
      make_pointbg (gray_bg, 128);
    }
  return FALSE;
}

static gboolean
  on_gray_drawingarea_motion_notify_event
  (GtkWidget * widget, GdkEventMotion * event, gpointer user_data) {
  return on_gray_drawingarea_button_press_event (widget,
                           (GdkEventButton *) event,
                           user_data);
}


static gboolean
on_gray_drawingarea_button_press_event (GtkWidget     *widget,
                                        GdkEventButton *event,
                                        gpointer        user_data)
{
  update_point (widget, &pv.gu, &pv.gv, event->x, event->y);
  update_values ();

  gtk_widget_queue_draw (widget);

  return FALSE;
}


static gboolean
on_white_drawingarea_button_press_event (GtkWidget      *widget,
                                         GdkEventButton *event,
                                         gpointer        user_data)
{
  update_point (widget, &pv.wu, &pv.wv, event->x, event->y);
  update_values ();

  gtk_widget_queue_draw (widget);

  return FALSE;
}


static gboolean
on_white_drawingarea_configure_event (GtkWidget         *widget,
                                      GdkEventConfigure *event,
                                      gpointer           user_data)
{
  if (!white_bg)
    {
      white_bg = g_malloc (128 * 128 * 3);
      make_pointbg (white_bg, 255);
    }
  return FALSE;
}


static gboolean
on_white_drawingarea_expose_event (GtkWidget      *widget,
                                   GdkEventExpose *event,
                                   gpointer        user_data)
{
  gdk_draw_rgb_image (widget->window, widget->style->white_gc,
            0, 0, 128, 128, GDK_RGB_DITHER_NONE, white_bg,
            128 * 3);

  gdk_draw_rectangle (widget->window, widget->style->black_gc,
            TRUE, (pv.wu / 2) + 64, 0, 1, 128);

  gdk_draw_rectangle (widget->window, widget->style->black_gc,
            TRUE, 0, 64 - (pv.wv / 2), 128, 1);
  return FALSE;
}


static gboolean
on_white_drawingarea_motion_notify_event (GtkWidget      *widget,
                                          GdkEventMotion *event,
                                          gpointer        user_data)
{
  return on_white_drawingarea_button_press_event
            (widget, (GdkEventButton *) event, user_data);
}



static gboolean
on_vectorscope_drawingarea_expose_event (GtkWidget      *widget,
                                         GdkEventExpose *event,
                                         gpointer        user_data)
{
  if (pp.vectorscope_enable && pp.prev_in_image)
    {
      gdk_draw_rgb_image (widget->window, widget->style->white_gc,
                0, 0, 128, 128, GDK_RGB_DITHER_NONE,
                vectorscope_data, 128 * 3);

      gdk_draw_rectangle (widget->window, widget->style->white_gc,
                TRUE, 64, 0, 1, 128);

      gdk_draw_rectangle (widget->window, widget->style->white_gc,
                TRUE, 0, 64, 128, 1);
    }
  else
    {
      gdk_draw_rectangle (widget->window, widget->style->black_gc,
                TRUE, 0, 0, 128, 128);
    }
  return FALSE;
}


static gboolean
on_vectorscope_drawingarea_configure_event (GtkWidget         *widget,
                                            GdkEventConfigure *event,
                                            gpointer           user_data)
{
  if (!vectorscope_data)
    {
      vectorscope_data = g_malloc (128 * 128 * 3);
      memset (vectorscope_data, 255, 128 * 128 * 3);
    }

  if (pp.vectorscope_enable && pp.prev_in_image)
    update_image_preview ();
  return FALSE;
}
